/*
	EditWaveCmd.cc

	Will 18, 2001.
*/


#include <DMenuCmd.h>
#include "OpResources.h"
#include "EditWaveCmd.h"
#include "AppWindow.h"
#include "GraphIntCmd.h"
#include "ViewWaveCmd.h"
#include <WorkingDialogManager.h>
#include <MessageDialogManager.h>
#include <InfoDialogManager.h>
#include <FileAccess.h>
#include "MixwaveUtils.h"

enum OPTYPE { REMOVE, INSERT };


EditWaveCmd::EditWaveCmd ( const char *name, int active, int opType ) : NoUndoCmd ( name, active )
{
	if ( ( opType < REMOVE ) || ( opType > INSERT ) )
		return;

	_opType = opType;		
	_srcWaveNum = 0;
	_dstWaveNum = 0;
	_popupRealised = False;
}


void EditWaveCmd::doCommand()
{
	if ( _popupRealised == TRUE )
		return;

	unsigned int n;
	unsigned int maxNameLength = 0;
	char msg[30];
	
	if ( (*wavePak)[0].pWave->first() == NULL )
		return;

	_arrayItems = wavePak->Count();

	for ( n = 0; n < _arrayItems; n++ )
		if ( strlen ( (*wavePak)[n].pWave->name() ) > maxNameLength )
			maxNameLength = strlen ( (*wavePak)[n].pWave->name() );

	n = 0;
	maxNameLength++;
	
	if ( maxNameLength < 5 )
		maxNameLength = 5;	// accomodate 'quit' at a minimum
		
	_arrayItems++;
	
	char strings [ _arrayItems ] [ maxNameLength ];

	for ( n = 0; n < ( _arrayItems - 1 ); n++ )
		strcpy ( ( char * ) &strings [ n ], (*wavePak)[n].pWave->name() );

	strcpy ( strings [ n ], "Quit" );

	if ( _opType == REMOVE )
		strcpy ( msg, "Select wave to cut" );
		
	if ( _opType == INSERT )
		strcpy ( msg, "Select source to copy" );

	_srcMenu = new DMenuCmd ( msg, TRUE, TRUE,
								&EditWaveCmd::srcMenuSelectionCallback,
								( void * ) this, "Current wave files",
								_arrayItems, maxNameLength, ( char * ) &strings );

	_srcMenu->execute();
	_popupRealised = True;
}


void EditWaveCmd::srcMenuSelectionCallback ( void *clientData, unsigned int selectedItemNum, char *label )
{
	EditWaveCmd *obj = ( EditWaveCmd * ) clientData;				

	if ( selectedItemNum < obj->_arrayItems )	// 'quit' selected
	{
		if ( obj->_opType == REMOVE )
		{
			obj->_srcWaveNum = selectedItemNum;
			obj->edit ( selectedItemNum );
		}

		if ( obj->_opType == INSERT )
			obj->selectTarget ( selectedItemNum );
	}

	delete obj->_srcMenu;
	obj->_popupRealised = False;
}


void EditWaveCmd::selectTarget ( unsigned int selectedItemNum )
{
	if ( selectedItemNum == 0 )
		return;
		
	unsigned int n;
	unsigned int maxNameLength = 0;
	char buf[1024];
			
	_srcWaveNum = selectedItemNum;
	_arrayItems = wavePak->Count();

	for ( n = 0; n < _arrayItems; n++ )
		if ( strlen ( (*wavePak)[n].pWave->name() ) > maxNameLength )
			maxNameLength = strlen ( (*wavePak)[n].pWave->name() );

	n = 0;
	maxNameLength++;
	
	if ( maxNameLength < 5 )
		maxNameLength = 5;
		
	_arrayItems++;
	
	char strings [ _arrayItems ] [ maxNameLength ];

	for ( n = 0; n < ( _arrayItems - 1 ); n++ )
		strcpy ( ( char * ) &strings [ n ], (*wavePak)[n].pWave->name() );

	strcpy ( strings [ n ], "Quit" );
	snprintf ( buf, sizeof ( buf ), "Source %s", (*wavePak)[_srcWaveNum-1].pWave->name() );	
	buf [ sizeof ( buf ) - 1 ] = 0;
	
	_dstMenu = new DMenuCmd ( "Select target for paste", TRUE, TRUE,
								&EditWaveCmd::dstMenuSelectionCallback,
								( void * ) this, buf,
								_arrayItems, maxNameLength, ( char * ) &strings );

	_dstMenu->execute();
}


void EditWaveCmd::dstMenuSelectionCallback ( void *clientData, unsigned int selectedItemNum, char *label )
{
	EditWaveCmd *obj = ( EditWaveCmd * ) clientData;				

	if ( selectedItemNum < obj->_arrayItems )	// 'quit menu' selected
		obj->edit ( selectedItemNum );

	delete obj->_dstMenu;
	obj->_popupRealised = False;
}


void EditWaveCmd::edit ( unsigned int selectedItemNum )
{
	if ( selectedItemNum == 0 )
		return;

	char msg[1024];

	_dstWaveNum = selectedItemNum;
	msg[0] = 0;

	if ( _opType == REMOVE )
	{
		if ( _dstWaveNum == 0 )
			return;
		
		if ( (*wavePak)[_dstWaveNum-1].startSel == (*wavePak)[_dstWaveNum-1].endSel )
		{
			strcpy ( msg, "No selection to cut." );
			messageDialogManager->post ( msg );
			return;
		}
	}

	if ( _opType == INSERT )
	{
		if ( ( _srcWaveNum == 0 ) || ( _dstWaveNum == 0 ) )
			return;	// give up on no source, no target or 'quit'

		if ( ( _srcWaveNum == _dstWaveNum ) && ( _opType == INSERT ) )
		{
			strncpy ( msg, "Cannot copy wave to itself.\nPlease create a new empty file\n\
as the target for the copy operation.", sizeof ( msg ) );
			msg [ sizeof ( msg ) - 1 ] = 0;
			messageDialogManager->post ( msg );
			return;
		}	

		if ( (*wavePak)[_srcWaveNum-1].startSel == (*wavePak)[_srcWaveNum-1].endSel )
		{
			strcpy ( msg, "No selection to copy." );
			messageDialogManager->post ( msg );
			return;
		}

// insert check for wavType and concatenate results?
		
		if ( mixwaveResources->enableWarningPopups() == True )
		{
			// no type check, assume non-linear-wav stuff doesn't make it this far
			if ( (*wavePak)[_srcWaveNum-1].pWave->numChannels() != (*wavePak)[_dstWaveNum-1].pWave->numChannels() )
			{
				snprintf ( msg, sizeof ( msg ), "%s channels %d\n%s channels %d\nChannel number incompatibility.",
							(*wavePak)[_srcWaveNum-1].pWave->name(),(*wavePak)[_srcWaveNum-1].pWave->numChannels(),
							(*wavePak)[_dstWaveNum-1].pWave->name(),(*wavePak)[_dstWaveNum-1].pWave->numChannels());
				msg [ sizeof ( msg ) - 1 ] = 0;
				messageDialogManager->post ( msg );
			}

			if ( (*wavePak)[_srcWaveNum-1].pWave->bitsPerSample() != (*wavePak)[_dstWaveNum-1].pWave->bitsPerSample() )
			{
				snprintf ( msg, sizeof ( msg ), "%s bits %d\n%s bits %d\nBits-per-sample incompatibility.",
							(*wavePak)[_srcWaveNum-1].pWave->name(),(*wavePak)[_srcWaveNum-1].pWave->bitsPerSample(),
							(*wavePak)[_dstWaveNum-1].pWave->name(),(*wavePak)[_dstWaveNum-1].pWave->bitsPerSample());
				msg [ sizeof ( msg ) - 1 ] = 0;
				messageDialogManager->post ( msg );
			}

			if ( (*wavePak)[_srcWaveNum-1].pWave->sampleRate() != (*wavePak)[_dstWaveNum-1].pWave->sampleRate() )
			{
				snprintf ( msg, sizeof ( msg ), "%s rate %ld\n%s rate %ld\nSample rate incompatibility.",
							(*wavePak)[_srcWaveNum-1].pWave->name(),(*wavePak)[_srcWaveNum-1].pWave->sampleRate(),
							(*wavePak)[_dstWaveNum-1].pWave->name(),(*wavePak)[_dstWaveNum-1].pWave->sampleRate());
				msg [ sizeof ( msg ) - 1 ] = 0;
				messageDialogManager->post ( msg );
			}
		}
	}

	ChangeWaveFileCmd *cwfCmd = new ChangeWaveFileCmd ( "WaveChange", TRUE, _srcWaveNum, _dstWaveNum, _opType );

	cwfCmd->execute ( &EditWaveCmd::changeWaveFileCmdFinishedCallback, ( void * ) this );
}


void EditWaveCmd::changeWaveFileCmdFinishedCallback ( BreakCmd *cmd, Boolean interrupted, void *clientData )
{
	ChangeWaveFileCmd *cwfObj = ( ChangeWaveFileCmd * ) cmd;
	EditWaveCmd *obj = ( EditWaveCmd * ) clientData;
	
	if ( interrupted )
		messageDialogManager->post ( "Interrupted!" );
	else
		obj->changeWaveFileCmdFinished ( cwfObj );
}


void EditWaveCmd::changeWaveFileCmdFinished ( ChangeWaveFileCmd *cwfObj )
{	
	if ( ( (*wavePak)[cwfObj->dstWaveNum()-1].pWave->dataLength() == 0 )
		|| ( mixwaveResources->enableWarningPopups() == False ) )	// new file or don't care
	{
		overwrite ( cwfObj->srcWaveNum(), cwfObj->dstWaveNum(), cwfObj->savedFile() );	// just do it
		delete cwfObj;
	}
	else
	{
		char msg[BUFSIZE<<1];
		snprintf ( msg, sizeof ( msg ), "%s\nReally overwrite this file?\nCancel to save as\n%s",
					(*wavePak)[_dstWaveNum-1].pWave->name(), cwfObj->savedFile() );
		msg [ sizeof ( msg ) - 1 ] = 0;
		infoDialogManager->post ( msg, cwfObj, &EditWaveCmd::overwriteOkCallback,
									&EditWaveCmd::overwriteCancelCallback );
	}
}


void EditWaveCmd::overwriteOkCallback ( void *clientData )
{
	ChangeWaveFileCmd *cwfCmd = ( ChangeWaveFileCmd * ) clientData;
	// typecast to get at overwrite method
	EditWaveCmd *ewc = ( EditWaveCmd * ) clientData;

	// note we can't use EditWaveCmd data directly after this point (e.g. _opType)
	ewc->overwrite ( cwfCmd->srcWaveNum(), cwfCmd->dstWaveNum(), cwfCmd->savedFile() );
	delete cwfCmd;
}


void EditWaveCmd::overwriteCancelCallback ( void *clientData )
{
	ChangeWaveFileCmd *cwfCmd = ( ChangeWaveFileCmd * ) clientData;

	delete cwfCmd;
}


void EditWaveCmd::overwrite ( unsigned int srcWaveNum, unsigned int dstWaveNum, char *fileName )
{
	// this is called by a dummy object which doesn't relate to any of the
	// edit objects instantiated via the AppWindow.cc menu construction
	// so _opType and other internal data is invalid here
	if ( (*wavePak)[dstWaveNum-1].fLock == ACTIVE )
		return;
			
	char command[BUFSIZE<<1];
	long startSel, endSel;
	int error = 0;
	
	snprintf ( command, sizeof ( command ), "%s %s %s", mixwaveResources->moveCmd(),
				fileName, (*wavePak)[dstWaveNum-1].pWave->name() );
	command [ sizeof ( command ) - 1 ] = 0;

	MixwaveUtils *utils = new MixwaveUtils;
	error = utils->sysCall ( command );
	delete utils;

	if ( error != 0 )
	{
		char buf[BUFSIZE*3];
		snprintf ( buf, sizeof ( buf ), "EditWaveCmd 'overwrite': system call failed\n%s\n\n", command );
		buf [ sizeof ( buf ) - 1 ] = 0;

		FileAccess *file = new FileAccess ( (*wavePak)[dstWaveNum-1].pWave->name(), "r+" );

		if ( file->errNum() != 0 )
			strncat ( buf, file->error(), sizeof ( buf ) - strlen ( buf ) - 1 );

		delete file;		
		messageDialogManager->post ( buf );
		return;
	}		

	(*wavePak)[dstWaveNum-1].pWave->LoadHeader ( (*wavePak)[dstWaveNum-1].pWave->name() );
	(*wavePak)[dstWaveNum-1].pWave->LoadBlock ( 0, (*wavePak)[dstWaveNum-1].pWave->dataLength());
	
	if ( (*wavePak)[dstWaveNum-1].graphCmd != NULL )	// old wave view exists and something to show
	{
		(*wavePak)[dstWaveNum-1].graphCmd->setDataSource ( (*wavePak)[dstWaveNum-1].pIntArray,
															(*wavePak)[dstWaveNum-1].pWave->dataLength()>>1,
															(*wavePak)[dstWaveNum-1].pWave->dataLength()>>1 );

		if ( srcWaveNum > 0 )	// INSERT type operation, 2 files involved
		{
			// start selection is insert point, end selection source sel end + insert point
			startSel = (*wavePak)[dstWaveNum-1].startSel;
			endSel = ( (*wavePak)[srcWaveNum-1].endSel - (*wavePak)[srcWaveNum-1].startSel ) + (*wavePak)[dstWaveNum-1].startSel;
		}
		else
			startSel = endSel = 0;

		if ( (*wavePak)[dstWaveNum-1].pWave->dataLength() > 0 )
		{
			(*wavePak)[dstWaveNum-1].graphCmd->redisplay ( 0L, (*wavePak)[dstWaveNum-1].pWave->dataLength()>>1,
															startSel, endSel, 0L, 0L );

			if ( (*wavePak)[dstWaveNum-1].waveDataCmd != NULL )	// old time view exists
				(*wavePak)[dstWaveNum-1].waveDataCmd->setFields ( 0L,
					(*wavePak)[dstWaveNum-1].pWave->dataLength()>>1, startSel, endSel, 0L, 0L,
					(*wavePak)[dstWaveNum-1].pWave->dataLength()>>1 );
		}
	}
}
